%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Program do obrazowania wynikw transformaty Z
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
clear;
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% W tym miejscu ustalamy posta transformaty Z dla rnych sygnaw:
% 0 -   delty Kroneckera - same jedynki
% 1 -   delty Kroneckera opnionej o n0 - n0-krotny biegun w zerze
% 2 -   rzeczywistego sygnau wykadniczego
%       x(n)=ampl^n - jeden biegun rzeczywisty
% 3 -   zespolonego sygnau wykadniczego o module ampl
%       x(n)=exp(1i*omega*n) - jeden biegun zespolony
% 4 -   zespolonego sygnau wykadniczego o module ampl
%       x(n)=ampl^n*exp(1i*omega*n) - jeden biegun zespolony
% 5 -   rzeczywistego sygnau harmonicznego o staej amplitudzie 
%       x(n)=sin(omega*n) - dwa sprzone bieguny zespolone
% 6 -   rzeczywistego sygnau harmonicznego o malejcej wykadniczo
%       amplitudzie
%       x(n)=ampl^n*sin(omega*n) - dwa sprzone bieguny zespolone
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
POSTAC=0;
% opnienie delty Kroneckera
n0=10;
% amplituda sygnaw
ampl=0.8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parametry wycinka paszczyzny Z
min_r=-2;
min_i=-2;
maks_r=2;
maks_i=2;
krok=0.0025;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ilo prbek postaci czasowej sygnau
ileprob=401;
% tworzymy wektor prbek dla postaci czasowej sygnau
ip2=floor(ileprob/2);
nrprob=-ip2:ip2;
disp('Czekaj obliczam...');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% definiujemy osie Re(Z) i Im(Z)
rez=min_r:krok:maks_r;
imz=min_i:krok:maks_i;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% tworzymy paszczyzn Z
ile_pkt_r=size(rez,2);
ile_pkt_i=size(imz,2);
for nr_r=1:ile_pkt_r
    for nr_i=1:ile_pkt_i
        z(nr_r,nr_i)=rez(nr_r)+1i*imz(nr_i);
    end;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% definiujemy potrzebne sygnay
% same jedynki
jed=ones(size(z));
% biegun rzeczywisty
ampl0=ampl;
% bieguny zespolone
% moduy
ampl1=ampl;
ampl2=ampl;
% argumenty - pulsacje
omega=0.1*pi;   % dla bieguna sin
omega1=0.15*pi;  % dla bieguna exp
omega2=-0.12*pi; % dla bieguna exp
omega3=0.15*pi;  % dla zera sin
omega4=-0.8*pi; % dla zera sin
a01=exp(omega1*1i);
a02=exp(-omega1*1i);
a1=ampl1*exp(omega1*1i);
a2=ampl2*exp(-omega2*1i);
% ustalamy domyln wysoko wykresu walca
wysokosc=8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch POSTAC 
    case 0
        Hz=jed;
        % dla tego przypadku wysoko musi by mniejsza
        s=0.^nrprob;
        tekst='h(n)=\delta(n)';
    case 1
        Hz=1./z.^n0;
        s=0.^(nrprob-n0);
        tekst='h(n)=\delta(n-n_0)';
    case 2
        Hz=z./(z-ampl0);
        s=ampl0.^nrprob;
        s(nrprob<0)=0;
        tekst='h(n)=a^n';
    case 3
        Hz=z./(z-a01);
        s=a01.^nrprob;
        s(nrprob<0)=0;
        tekst='h(n)=e^{j\Omega_0n}';
    case 4
        Hz=z./(z-a01);
        s=a1.^nrprob;
        s(nrprob<0)=0;
        tekst='h(n)=a^ne^{j\Omega_0n}';
    case 5
        Hz=z./(z-a01)-z./(z-a02);
        s=0.5*(a01.^nrprob+a02.^nrprob);
        s(nrprob<0)=0;
        tekst='h(n)=sin(\Omega_0n)';
    case 6
        Hz=z./(z-a1)-z./(z-a2);
        s=0.5*(a1.^nrprob+a2.^nrprob);
        s(nrprob<0)=0;
        tekst='h(n)=a^nsin(\Omega_0n)';
    otherwise
        Hz=jed;
        % dla tego przypadku wysoko musi by mniejsza
        wysokosc=2;
        s=zeros(size(nrprob));;
        tekst('');
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rysujemy posta czasow sygnau
figure;
maxwindow;
plot(nrprob,real(s),'b--');
if(max(abs(imag(s)>0)))
    hold on;
    plot(nrprob,imag(s),'r--');
    legend('Re[h(n)]','Im[h(n)]');
end;    % koniec dla if(max(abs(imag(s)>0)))
ylim([-1.5 1.5]);
xlabel('n');
ylabel('h(n)')
title(['Posta czasowa sygnau ' tekst]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wyliczamy modu funkcji H(z) i ograniczamy wysoko
absHz=limituj(abs(Hz),0,wysokosc);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wykres paski 
figure;
maxwindow;
imagesc(rez, imz, absHz);
colormap('jet')
axis equal
xlabel('Re(z)');
ylabel('jIm(z)');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wykres przestrzenny
figure;
maxwindow;
surf(rez, imz, absHz', 'linestyle','none');
xlim([min_r maks_r]);
ylim([min_i maks_i]);
xlabel('jIm(z)');
ylabel('Re(z)');
zlabel('|H(z)|')
colormap('jet')
axis equal
hold on;
% dorysowanie walca jednostkowego
[X,Y,Z] = cylinder(1,250);
% pytamy uytkownika o zgod
clc
odp=input('Rysowa walec? ','s');
if(odp=='t'||odp=='T')
    % ... i rysujemy
    surf(X,Y,Z.*wysokosc,'FaceAlpha',0.71,'linestyle','none');
end;
hold off;
dw=0.05;
wmax=360-dw;
nr_w=0;
w1=0:dw:wmax;
lHz=zeros(1,size(w1,2));
wHz=zeros(size(Hz));
for w=0:dw:wmax-dw
    nr_w=nr_w+1;
    rz(nr_w)=cos(deg2rad(w));
    iz(nr_w)=sin(deg2rad(w));
    nr_r(nr_w)=floor(2+(rz(nr_w)-min_r)/krok);
    nr_i(nr_w)=floor(2+(iz(nr_w)-min_i)/krok);
    if((nr_r(nr_w)<ile_pkt_r)&&(nr_i(nr_w)<ile_pkt_i))
        lHz(nr_w)=absHz(nr_r(nr_w),nr_i(nr_w));
        wHz(nr_r(nr_w),nr_i(nr_w))=absHz(nr_r(nr_w),nr_i(nr_w));
    end;
end;
lHz(end)=lHz(1);
w2=deg2rad(w1-180);
figure;
maxwindow;
plot(w2,(fftshift(lHz)));%,'LineWidth',1.5);
xlabel('\Omega');
ylabel('|H(e^(^j^\Omega^))|');
title(['Widmo sygnau ' tekst '. Transmitancja czstotliwociowa ukadu']);
maxy=max(lHz);
ylim([0 8]);
figure;
maxwindow;
surf(rez, imz, wHz', 'linestyle','none');
xlabel('Im(z)');
ylabel('Re(z)');
